package Text::CSV::Parse;

use strict;
use Carp qw(croak);
sub DIAG() { 0 }

sub START () { 0 }
sub RNS   () { 1 } # Read unquoted
sub RQS   () { 2 } # Read quoted
sub RQ    () { 3 }

sub SEP   () { ';' }

sub E_SAVE()  { 2**1 }
sub E_BUF()   { 2**2 }
sub E_ERROR() { 2**3 }

sub STT   () {{
	START,{ SEP, [ START, E_SAVE  ],
	        '"', [ RQS,   undef   ],
	        '',  [ RNS,   E_BUF   ]},
	
	RNS,  { SEP, [ START, E_SAVE  ],
	        '"', [ undef, E_ERROR ],
	        '',  [ RNS,   E_BUF   ]},
	
	RQS,  { '"', [ RQ,    undef   ],
	        '',  [ RQS,   E_BUF   ]},
	
	RQ,   { SEP, [ START, E_SAVE  ],
	        '"', [ RQS,   E_BUF   ],
	        '',  [ undef, E_ERROR ]},
}}

sub parse_line {
	local $_ = @_ ? shift : $_;
	my $st  = START;
	my $res = [''];
	chomp;
	my $char = 0;
	for (split //) {
		$char++;
		my ($new,$ev) = @{ STT->{$st}{$_} || STT->{$st}{''} };
		warn "$_ : $st [ $new $ev ]\n" if DIAG and DIAG > 3;
		if (defined $new) {
			$st = $new;
		}
		if (defined $ev) {
			if ($ev & E_BUF) {
				$$res[-1] .= $_;
			}
			if ($ev & E_SAVE) {
				push @$res,'';
			}
			if ($ev & E_ERROR) {
				warn "Malformed string\n" if DIAG;
				return [];
			}
		}
	}
	return $res;
}

sub parse(;$) {
	local $_ = @_ ? shift : $_;
	return [ map parse_line, split /\r?\n/ ];
}

1;
